import os
import re
from models import Book, File, Gif, Chapter
from app import db
from backend_flask.services.file_manipulation import PDFService

BASE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))

UPLOAD_FOLDER = os.path.join(BASE_DIR, 'images')
UPLOAD_FOLDER_FILES = os.path.join(BASE_DIR, 'files')
UPLOAD_FOLDER_GIFS = os.path.join(BASE_DIR, 'gifs')

def save_cover(cover):
    cover_filename = cover.filename
    cover_path = os.path.join(UPLOAD_FOLDER, cover_filename)
    cover.save(cover_path)

    return cover_filename


def save_pdf(pdf, book_id):
    try:
        pdf_name, pdf_filename = save_pdf_to_filesystem(pdf)
        save_file_to_db(book_id, pdf_name, pdf_filename)
    except Exception as e:
        raise ValueError("The PDF file could not be processed.")

    return pdf_filename


def save_gif(gif, description):
    gif_filename = gif.filename
    gif_name_without_extension = os.path.splitext(gif_filename)[0]


    if not gif_filename.lower().endswith('.gif'):
        raise ValueError("File is not a valid GIF.")

    gif_path = os.path.join(UPLOAD_FOLDER_GIFS, gif_filename)

    try:

        gif.save(gif_path)
    except Exception as e:
        raise ValueError("Error saving the GIF. The file may be corrupted.")

    return gif_name_without_extension, gif_filename, description



def save_pdf_to_filesystem(pdf):
    pdf_filename = pdf.filename
    pdf_name_without_extension = os.path.splitext(pdf_filename)[0]
    pdf_path = os.path.join(UPLOAD_FOLDER_FILES, pdf_filename)

    pdf.save(pdf_path)

    return pdf_name_without_extension, pdf_filename


def extract_and_save_chapters(pdf_filename, book_id):
    try:
        num_pages, pdf_content = PDFService.extract_pdf_info(pdf_filename)
    except Exception as e:
        raise ValueError("Error extracting chapters from PDF. The file might be corrupted.")

    chapters = []
    chapter_number = 1
    chapter_matches = re.finditer(r'<h1>(.*?)<\/h1>', pdf_content)

    for match in chapter_matches:
        title = match.group(1)
        start_page = get_start_page_for_chapter(chapter_number)
        end_page = get_end_page_for_chapter(chapter_number)

        chapter = Chapter(
            book_id=book_id,
            chapter_number=chapter_number,
            title=title,
            start_page=start_page,
            end_page=end_page
        )

        chapters.append(chapter)
        chapter_number += 1

    db.session.bulk_save_objects(chapters)
    db.session.commit()

    return num_pages


def get_start_page_for_chapter(chapter_number):
    return chapter_number

def get_end_page_for_chapter(chapter_number):
    return chapter_number + 1

def save_book_to_db(category_id, title, author, description, total_pages, published_at, cover_filename):
    new_book = Book(
        category_id=category_id,
        title=title,
        author=author,
        description=description,
        total_pages=total_pages,
        published_at=published_at,
        cover=cover_filename
    )

    db.session.add(new_book)
    db.session.commit()

    return new_book



def save_file_to_db(book_id, pdf_name, pdf_filename):
    new_file = File(
        book_id=book_id,
        name=pdf_name,
        file_url=pdf_filename
    )

    db.session.add(new_file)
    db.session.commit()

    return new_file

def save_gifs_to_db(book_id, gifs):

    book = Book.query.get(book_id)
    if not book:
        return {'error': 'Book not found.'}, 404


    for gif, description in gifs:
        gif_name, gif_filename, description = save_gif(gif, description)
        new_gif = Gif(
            book_id=book_id,
            name=gif_name,
            gif_url=gif_filename,
            description=description
        )
        db.session.add(new_gif)

    db.session.commit()

    return {'status': 'success'}, 201